@@ -30,9 +30,9 @@
 #include <ktoolbarpopupaction.h>
 #include <kactionmenu.h>
 #include <ktoggleaction.h>
-#include <kselectaction.h>
 
 #include <QCursor>
+#include <QDesktopServices>
 #include <QDir>
 #include <QDirIterator>
 #include <QHash>
@@ -40,6 +40,7 @@
 #include <QFile>
 #include <QFileDialog>
 #include <QHeaderView>
+#include <QList>
 #include <QResizeEvent>
 #include <QMouseEvent>
 #include <QKeyEvent>
@@ -65,27 +66,29 @@
 #include <cmath>
 #include <algorithm>
 #include <random>
+#include <utility>
 
+#include "actioncollection.h"
+#include "cache.h"
+#include "collectionlist.h"
+#include "coverdialog.h"
+#include "coverinfo.h"
+#include "deletedialog.h"
 #include "directoryloader.h"
-#include "playlistitem.h"
+#include "filerenamer.h"
+#include "iconsupport.h"
+#include "juk_debug.h"
+#include "juktag.h"
+#include "mediafiles.h"
 #include "playlistcollection.h"
+#include "playlistitem.h"
 #include "playlistsearch.h"
 #include "playlistsharedsettings.h"
-#include "mediafiles.h"
-#include "collectionlist.h"
-#include "filerenamer.h"
-#include "actioncollection.h"
-#include "juktag.h"
-#include "upcomingplaylist.h"
-#include "deletedialog.h"
-#include "webimagefetcher.h"
-#include "coverinfo.h"
-#include "coverdialog.h"
 #include "tagtransactionmanager.h"
-#include "cache.h"
-#include "juk_debug.h"
+#include "upcomingplaylist.h"
+#include "webimagefetcher.h"
 
-using namespace ActionCollection;
+using namespace ActionCollection; // ""_act and others
 
 /**
  * Used to give every track added in the program a unique identifier. See
@@ -99,7 +102,7 @@ quint32 g_trackID = 0;
 
 static bool manualResize()
 {
-    return action<KToggleAction>("resizeColumnsManually")->isChecked();
+    return "resizeColumnsManually"_act->isChecked();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -120,11 +123,11 @@ Playlist::Playlist(
         bool delaySetup, const QString &name,
         PlaylistCollection *collection, const QString &iconName,
         int extraCols)
-    : QTreeWidget(collection->playlistStack())
-    , m_collection(collection)
-    , m_playlistName(name)
-    , m_refillDebounce(new QTimer(this))
-    , m_fetcher(new WebImageFetcher(this))
+  : QTreeWidget(collection->playlistStack())
+  , m_collection(collection)
+  , m_playlistName(name)
+  , m_refillDebounce(new QTimer(this))
+  , m_fetcher(new WebImageFetcher(this))
 {
     setup(extraCols);
 
@@ -139,7 +142,7 @@ Playlist::Playlist(
     // group to emit the triggered signal.
     QActionGroup *randomGroup = action("disableRandomPlay")->actionGroup();
     connect(randomGroup,      &QActionGroup::triggered,
-            m_refillDebounce, QOverload<>::of(&QTimer::start));
+            m_refillDebounce, qOverload<>(&QTimer::start));
 
     // Some subclasses need to do even more handling but will remember to
     // call setupPlaylist
@@ -342,10 +345,11 @@ void Playlist::save()
 
     QTextStream stream(&file);
 
-    QStringList fileList = files();
+    const QStringList fileList = files();
 
-    foreach(const QString &file, fileList)
+    for(const auto &file : fileList) {
         stream << file << '\n';
+    }
 
     file.close();
 }
@@ -383,15 +387,17 @@ void Playlist::clearItem(PlaylistItem *item)
 
 void Playlist::clearItems(const PlaylistItemList &items)
 {
-    foreach(PlaylistItem *item, items)
+    for(auto &item : items) {
         delete item;
-
+    }
     playlistItemsChanged();
 }
 
 PlaylistItem *Playlist::playingItem() // static
 {
-    return PlaylistItem::playingItems().isEmpty() ? 0 : PlaylistItem::playingItems().front();
+    return PlaylistItem::playingItems().isEmpty()
+        ? nullptr
+        : PlaylistItem::playingItems().front();
 }
 
 QStringList Playlist::files() const
@@ -504,10 +510,10 @@ void Playlist::synchronizePlayingItems(const PlaylistList &sources, bool setMast
 
 void Playlist::copy()
 {
-    PlaylistItemList items = selectedItems();
+    const PlaylistItemList items = selectedItems();
     QList<QUrl> urls;
 
-    foreach(PlaylistItem *item, items) {
+    for(const auto &item : items) {
         urls << QUrl::fromLocalFile(item->file().absFilePath());
     }
 
@@ -535,12 +541,12 @@ void Playlist::clear()
 
 void Playlist::slotRefresh()
 {
-    PlaylistItemList l = selectedItems();
-    if(l.isEmpty())
-        l = visibleItems();
+    PlaylistItemList itemList = selectedItems();
+    if(itemList.isEmpty())
+        itemList = visibleItems();
 
     QApplication::setOverrideCursor(Qt::WaitCursor);
-    foreach(PlaylistItem *item, l) {
+    for(auto &item : itemList) {
         item->refreshFromDisk();
 
         if(!item->file().tag() || !item->file().fileInfo().exists()) {
@@ -554,6 +560,39 @@ void Playlist::slotRefresh()
     QApplication::restoreOverrideCursor();
 }
 
+void Playlist::slotOpenItemDir()
+{
+    PlaylistItemList itemList = selectedItems();
+    QList<QUrl> pathList;
+
+    for(auto &item : itemList) {
+        QUrl path = QUrl::fromLocalFile(item->file().fileInfo().absoluteDir().absolutePath());
+        if(!pathList.contains(path))
+            pathList.append(path);
+    }
+
+    if (pathList.length() > 4) {
+        if(KMessageBox::warningContinueCancel(
+            this,
+            i18np("You are about to open directory. Are you sure you want to continue?",
+                  "You are about to open %1 directories. Are you sure you want to continue?",
+                  pathList.length()),
+            i18n("Open Containing Folder")
+        ) == KMessageBox::Cancel)
+        {
+            return;
+        }
+    }
+
+    QApplication::setOverrideCursor(Qt::WaitCursor);
+    for(auto &path : pathList) {
+        QDesktopServices::openUrl(path);
+
+        processEvents();
+    }
+    QApplication::restoreOverrideCursor();
+}
+
 void Playlist::slotRenameFile()
 {
     FileRenamer renamer;
@@ -589,10 +628,14 @@ void Playlist::slotBeginPlayback()
 void Playlist::slotViewCover()
 {
     const PlaylistItemList items = selectedItems();
-    if (items.isEmpty())
-        return;
-    foreach(const PlaylistItem *item, items)
-        item->file().coverInfo()->popup();
+    for(const auto &item : items) {
+        const auto cover = item->file().coverInfo();
+
+        if(cover->hasCover()) {
+            cover->popup();
+            return; // If we select multiple items, only show one
+        }
+    }
 }
 
 void Playlist::slotRemoveCover()
@@ -664,7 +707,7 @@ void Playlist::slotGuessTagInfo(TagGuesser::Type type)
 
     m_blockDataChanged = true;
 
-    foreach(PlaylistItem *item, items) {
+    for(auto &item : items) {
         item->guessTagInfo(type);
         processEvents();
     }
@@ -766,7 +809,7 @@ void Playlist::slotShowPlaying()
     m_collection->raise(l);
 
     l->setCurrentItem(playingItem());
-    l->scrollToItem(playingItem());
+    l->scrollToItem(playingItem(), QAbstractItemView::PositionAtCenter);
 }
 
 void Playlist::slotColumnResizeModeChanged()
@@ -798,47 +841,49 @@ void Playlist::playlistItemsChanged()
 
 void Playlist::removeFromDisk(const PlaylistItemList &items)
 {
-    if(isVisible() && !items.isEmpty()) {
+    if(!isVisible() || items.isEmpty()) {
+        return;
+    }
 
-        QStringList files;
-        foreach(const PlaylistItem *item, items)
-            files.append(item->file().absFilePath());
+    QStringList files;
+    for(const auto &item : items) {
+        files.append(item->file().absFilePath());
+    }
 
-        DeleteDialog dialog(this);
+    DeleteDialog dialog(this);
 
-        m_blockDataChanged = true;
+    m_blockDataChanged = true;
 
-        if(dialog.confirmDeleteList(files)) {
-            bool shouldDelete = dialog.shouldDelete();
-            QStringList errorFiles;
+    if(dialog.confirmDeleteList(files)) {
+        bool shouldDelete = dialog.shouldDelete();
+        QStringList errorFiles;
 
-            foreach(PlaylistItem *item, items) {
-                if(playingItem() == item)
-                    action("forward")->trigger();
+        for(const auto &item : items) {
+            if(playingItem() == item)
+                action("forward")->trigger();
 
-                QString removePath = item->file().absFilePath();
-                QUrl removeUrl = QUrl::fromLocalFile(removePath);
-                if((!shouldDelete && KIO::trash(removeUrl)->exec()) ||
-                   (shouldDelete && QFile::remove(removePath)))
-                {
-                    delete item->collectionItem();
-                }
-                else
-                    errorFiles.append(item->file().absFilePath());
-            }
-
-            if(!errorFiles.isEmpty()) {
-                QString errorMsg = shouldDelete ?
-                        i18n("Could not delete these files") :
-                        i18n("Could not move these files to the Trash");
-                KMessageBox::errorList(this, errorMsg, errorFiles);
+            QString removePath = item->file().absFilePath();
+            QUrl removeUrl = QUrl::fromLocalFile(removePath);
+            if((!shouldDelete && KIO::trash(removeUrl)->exec()) ||
+               (shouldDelete && QFile::remove(removePath)))
+            {
+                delete item->collectionItem();
             }
+            else
+                errorFiles.append(item->file().absFilePath());
         }
 
-        m_blockDataChanged = false;
-
-        playlistItemsChanged();
+        if(!errorFiles.isEmpty()) {
+            QString errorMsg = shouldDelete ?
+                    i18n("Could not delete these files") :
+                    i18n("Could not move these files to the Trash");
+            KMessageBox::errorList(this, errorMsg, errorFiles);
+        }
     }
+
+    m_blockDataChanged = false;
+
+    playlistItemsChanged();
 }
 
 void Playlist::synchronizeItemsTo(const PlaylistItemList &itemList)
@@ -918,7 +963,7 @@ bool Playlist::eventFilter(QObject *watched, QEvent *e)
             }
 
             if(!manualResize() && m_widthsDirty)
-                QTimer::singleShot(0, this, SLOT(slotUpdateColumnWidths()));
+                QTimer::singleShot(0, this, &Playlist::slotUpdateColumnWidths);
             break;
         }
         default:
@@ -957,8 +1002,8 @@ QStringList Playlist::mimeTypes() const
 QMimeData* Playlist::mimeData(const QList<QTreeWidgetItem *> items) const
 {
     QList<QUrl> urls;
-    foreach(QTreeWidgetItem *item, items) {
-        urls << QUrl::fromLocalFile(static_cast<PlaylistItem*>(item)->file().absFilePath());
+    for(const auto &item : items) {
+        urls << QUrl::fromLocalFile(static_cast<const PlaylistItem*>(item)->file().absFilePath());
     }
 
     QMimeData *urlDrag = new QMimeData();
@@ -993,7 +1038,7 @@ void Playlist::dropEvent(QDropEvent *e)
 
         if(item->isSelected()) {
             const PlaylistItemList selItems = selectedItems();
-            foreach(PlaylistItem *playlistItem, selItems) {
+            for(auto &playlistItem : selItems) {
                 playlistItem->file().coverInfo()->setCoverId(id);
                 playlistItem->refresh();
             }
@@ -1025,20 +1070,13 @@ void Playlist::dropEvent(QDropEvent *e)
         sortItems(columnCount() + 1, Qt::AscendingOrder);
 
         const QList<QTreeWidgetItem *> items = QTreeWidget::selectedItems();
+        int insertIndex = item ? indexOfTopLevelItem(item) : 0;
 
-        foreach(QTreeWidgetItem *listViewItem, items) {
-            if(!item) {
+        // Move items from elsewhere in the playlist
 
-                // Insert the item at the top of the list.  This is a bit ugly,
-                // but I don't see another way.
-
-                takeItem(listViewItem);
-                insertItem(listViewItem);
-            }
-            //else
-            //    listViewItem->moveItem(item);
-
-            item = static_cast<PlaylistItem *>(listViewItem);
+        for(auto &listViewItem : items) {
+            auto oldItem = takeTopLevelItem(indexOfTopLevelItem(listViewItem));
+            insertTopLevelItem(++insertIndex, oldItem);
         }
     }
     else
@@ -1086,7 +1124,7 @@ void Playlist::read(QDataStream &s)
 
     m_blockDataChanged = true;
 
-    foreach(const QString &file, files) {
+    for(const auto &file : qAsConst(files)) {
         if(file.isEmpty())
             throw BICStreamException();
 
@@ -1237,8 +1275,10 @@ void Playlist::refreshAlbums(const PlaylistItemList &items, coverKey id)
 
 void Playlist::updatePlaying() const
 {
-    foreach(const PlaylistItem *item, PlaylistItem::playingItems())
+    const auto playingItems = PlaylistItem::playingItems();
+    for(const auto &item : playingItems) {
         item->treeWidget()->viewport()->update();
+    }
 }
 
 void Playlist::refreshAlbum(const QString &artist, const QString &album)
@@ -1269,18 +1309,20 @@ void Playlist::refreshAlbum(const QString &artist, const QString &album)
 
 void Playlist::hideColumn(int c, bool updateSearch)
 {
-    foreach (QAction *action, m_headerMenu->actions()) {
+    const auto headerActions = m_headerMenu->actions();
+    for(auto &action : headerActions) {
         if(!action)
             continue;
 
-        if (action->data().toInt() == c) {
+        if(action->data().toInt() == c) {
             action->setChecked(false);
             break;
         }
     }
 
-    if(isColumnHidden(c))
+    if(isColumnHidden(c)) {
         return;
+    }
 
     QTreeWidget::hideColumn(c);
 
@@ -1294,27 +1336,31 @@ void Playlist::hideColumn(int c, bool updateSearch)
         viewport()->update();
     }
 
-    if(this != CollectionList::instance())
+    if(this != CollectionList::instance()) {
         CollectionList::instance()->hideColumn(c, false);
+    }
 
-    if(updateSearch)
+    if(updateSearch) {
         redisplaySearch();
+    }
 }
 
 void Playlist::showColumn(int c, bool updateSearch)
 {
-    foreach (QAction *action, m_headerMenu->actions()) {
+    const auto headerActions = m_headerMenu->actions();
+    for(auto &action : headerActions) {
         if(!action)
             continue;
 
-        if (action->data().toInt() == c) {
+        if(action->data().toInt() == c) {
             action->setChecked(true);
             break;
         }
     }
 
-    if(!isColumnHidden(c))
+    if(!isColumnHidden(c)) {
         return;
+    }
 
     QTreeWidget::showColumn(c);
 
@@ -1328,11 +1374,13 @@ void Playlist::showColumn(int c, bool updateSearch)
         viewport()->update();
     }
 
-    if(this != CollectionList::instance())
+    if(this != CollectionList::instance()) {
         CollectionList::instance()->showColumn(c, false);
+    }
 
-    if(updateSearch)
+    if(updateSearch) {
         redisplaySearch();
+    }
 }
 
 void Playlist::sortByColumn(int column, Qt::SortOrder order)
@@ -1399,10 +1447,11 @@ void Playlist::slotInitialize(int numColumnsToReserve)
         resizeColumnToContents(i);
     }
 
-    connect(m_headerMenu, SIGNAL(triggered(QAction*)), this, SLOT(slotToggleColumnVisible(QAction*)));
+    connect(m_headerMenu, &QMenu::triggered,
+            this, &Playlist::slotToggleColumnVisible);
 
-    connect(this, SIGNAL(customContextMenuRequested(QPoint)),
-            this, SLOT(slotShowRMBMenu(QPoint)));
+    connect(this, &QWidget::customContextMenuRequested,
+            this, &Playlist::slotShowRMBMenu);
 
     // Disabled for now because adding new items (File->Open) causes Qt to send
     // an itemChanged signal for unrelated playlist items which can cause the
@@ -1414,8 +1463,8 @@ void Playlist::slotInitialize(int numColumnsToReserve)
     //connect(this, &QTreeWidget::itemChanged,
     //        this, &Playlist::slotInlineEditDone);
 
-    connect(action("resizeColumnsManually"), SIGNAL(triggered()),
-            this, SLOT(slotColumnResizeModeChanged()));
+    connect(action("resizeColumnsManually"), &KToggleAction::triggered,
+            this, &Playlist::slotColumnResizeModeChanged);
 
     if(action<KToggleAction>("resizeColumnsManually")->isChecked()) {
         header()->setSectionResizeMode(QHeaderView::Interactive);
@@ -1526,15 +1575,19 @@ void Playlist::setup(int numColumnsToReserve)
     setUniformRowHeights(true);
     setEditTriggers(QAbstractItemView::EditKeyPressed); // Don't edit on double-click
 
-    connect(header(), SIGNAL(sectionMoved(int,int,int)), this, SLOT(slotColumnOrderChanged(int,int,int)));
+    connect(header(), &QHeaderView::sectionMoved,
+            this,     &Playlist::slotColumnOrderChanged);
 
-    connect(m_fetcher, SIGNAL(signalCoverChanged(int)), this, SLOT(slotCoverChanged(int)));
+    connect(m_fetcher, &WebImageFetcher::signalCoverChanged,
+            this,      &Playlist::slotCoverChanged);
 
     // Prevent list of selected items from changing while internet search is in
     // progress.
-    connect(this, SIGNAL(itemSelectionChanged()), m_fetcher, SLOT(abortSearch()));
+    connect(this,      &Playlist::itemSelectionChanged,
+            m_fetcher, &WebImageFetcher::abortSearch);
 
-    connect(this, &QTreeWidget::itemDoubleClicked, this, &Playlist::slotPlayCurrent);
+    connect(this, &QTreeWidget::itemDoubleClicked,
+            this, &Playlist::slotPlayCurrent);
 
     // Use a timer to soak up the multiple dataChanged signals we're going to get
     auto updateRequestor = new QTimer(this);
@@ -1542,8 +1595,9 @@ void Playlist::setup(int numColumnsToReserve)
     updateRequestor->setInterval(10);
 
     connect(model(), &QAbstractItemModel::dataChanged,
-            updateRequestor, static_cast<void (QTimer::*)()>(&QTimer::start));
-    connect(updateRequestor, &QTimer::timeout, this, &Playlist::slotUpdateTime);
+            updateRequestor, qOverload<>(&QTimer::start));
+    connect(updateRequestor, &QTimer::timeout,
+            this, &Playlist::slotUpdateTime);
 
     // This apparently must be created very early in initialization for other
     // Playlist code requiring m_headerMenu.
@@ -1601,7 +1655,6 @@ void Playlist::loadFile(const QString &fileName, const QFileInfo &fileInfo)
     playlistItemsChanged();
 }
 
-// static
 void Playlist::setPlaying(PlaylistItem *item, bool addToHistory)
 {
     auto wasPlayingItem = playingItem();
@@ -1622,6 +1675,15 @@ void Playlist::setPlaying(PlaylistItem *item, bool addToHistory)
 
     if(item) {
         item->setPlaying(true);
+
+        if(wasPlayingItem
+            && m_search->isEmpty()
+            && state() == QAbstractItemView::NoState
+            && (!m_rmbMenu || m_rmbMenu->isHidden())
+            )
+        {
+            scrollToItem(item, QAbstractItemView::PositionAtCenter);
+        }
     }
 }
 
@@ -1639,6 +1701,47 @@ int Playlist::leftMostVisibleColumn() const
     return i < PlaylistItem::lastColumn() ? i : 0;
 }
 
+// used with slotShowRMBMenu
+void Playlist::createPlaylistRMBMenu()
+{
+    using namespace IconSupport; // ""_icon
+
+    m_rmbMenu = new QMenu(this);
+
+    m_rmbMenu->addAction("go-jump-today"_icon, i18n("Add to Play Queue"),
+            this, &Playlist::slotAddToUpcoming);
+    m_rmbMenu->addSeparator();
+
+    if(!this->readOnly()) {
+        m_rmbMenu->addAction(action("edit_cut"));
+        m_rmbMenu->addAction(action("edit_copy"));
+        m_rmbMenu->addAction(action("edit_paste"));
+        m_rmbMenu->addSeparator();
+        m_rmbMenu->addAction(action("removeFromPlaylist"));
+    }
+    else
+        m_rmbMenu->addAction(action("edit_copy"));
+
+    m_rmbEdit = m_rmbMenu->addAction(i18n("Edit"));
+
+    m_rmbMenu->addAction(action("refresh"));
+    m_rmbMenu->addAction(action("openItemDir"));
+    m_rmbMenu->addAction(action("removeItem"));
+
+    m_rmbMenu->addSeparator();
+
+    m_rmbMenu->addAction(action("guessTag"));
+    m_rmbMenu->addAction(action("renameFile"));
+
+    m_rmbMenu->addAction(action("coverManager"));
+
+    m_rmbMenu->addSeparator();
+
+    m_rmbMenu->addAction("folder-new"_icon,
+        i18n("Create Playlist From Selected Items..."),
+        this, &Playlist::slotCreateGroup);
+}
+
 PlaylistItemList Playlist::items(QTreeWidgetItemIterator::IteratorFlags flags)
 {
     PlaylistItemList list;
@@ -1654,38 +1757,37 @@ void Playlist::calculateColumnWeights()
     if(m_disableColumnWidthUpdates)
         return;
 
-    const PlaylistItemList l = items();
+    const PlaylistItemList itemList = items();
 
     QVector<double> averageWidth(columnCount());
-    double itemCount = l.size();
-
-    QVector<int> cachedWidth;
+    double itemCount = itemList.size();
 
     // Here we're not using a real average, but averaging the squares of the
     // column widths and then using the square root of that value.  This gives
     // a nice weighting to the longer columns without doing something arbitrary
     // like adding a fixed amount of padding.
 
-    foreach(PlaylistItem *item, l) {
-        cachedWidth = item->cachedWidths();
+    for(const auto &item : itemList) {
+        const auto cachedWidth = item->cachedWidths();
 
         // Extra columns start at 0, but those weights aren't shared with all
         // items.
         for(int i = 0; i < columnOffset(); ++i) {
-            averageWidth[i] +=
-                std::pow(double(columnWidth(i)), 2.0) / itemCount;
+            const auto width = columnWidth(i);
+            averageWidth[i] += width * width / itemCount;
         }
 
-        for(int column = columnOffset(); column < columnCount(); ++column) {
-            averageWidth[column] +=
-                std::pow(double(cachedWidth[column - columnOffset()]), 2.0) / itemCount;
+        const auto offset = columnOffset();
+        for(int column = offset; column < columnCount(); ++column) {
+            const auto width = cachedWidth[column - offset];
+            averageWidth[column] += width * width / itemCount;
         }
     }
 
     if(m_columnWeights.isEmpty())
         m_columnWeights.fill(-1, columnCount());
 
-    foreach(int column, m_weightDirty) {
+    for(const auto column : qAsConst(m_weightDirty)) {
         m_columnWeights[column] = int(std::sqrt(averageWidth[column]) + 0.5);
     }
 
@@ -1694,7 +1796,7 @@ void Playlist::calculateColumnWeights()
 
 void Playlist::addPlaylistFile(const QString &m3uFile)
 {
-    if (!m_collection->containsPlaylistFile(m3uFile)) {
+    if(!m_collection->containsPlaylistFile(m3uFile)) {
         new Playlist(m_collection, QFileInfo(m3uFile));
     }
 }
@@ -1750,12 +1852,14 @@ QFuture<void> Playlist::addUntypedFile(const QString &file, PlaylistItem *after)
     }
 
     if(fileInfo.isDir()) {
-        foreach(const QString &directory, m_collection->excludedFolders()) {
-            if(canonicalPath.startsWith(directory))
-                return {}; // Exclude it
+        const auto blockedPaths = m_collection->excludedFolders();
+        if(std::none_of(blockedPaths.begin(), blockedPaths.end(),
+            [canonicalPath](const auto &dir) {
+                return canonicalPath.startsWith(dir);
+            }))
+        {
+            return addFilesFromDirectory(canonicalPath);
         }
-
-        return addFilesFromDirectory(canonicalPath);
     }
 
     return {};
@@ -1815,9 +1919,11 @@ void Playlist::slotUpdateColumnWidths()
 #endif
     };
 
+    // No item content to auto-fit around, use the headers for now
     if(count() == 0) {
-        foreach(int column, visibleColumns)
+        for(int column : qAsConst(visibleColumns)) {
             setColumnWidth(column, textWidth(header()->fontMetrics(),headerItem()->text(column)) + 10);
+        }
 
         return;
     }
@@ -1837,7 +1943,7 @@ void Playlist::slotUpdateColumnWidths()
     QVector<int> minimumFixedWidth(columnCount(), 0);
     int minimumFixedWidthTotal = 0;
 
-    foreach(int column, visibleColumns) {
+    for(int column : qAsConst(visibleColumns)) {
         minimumWidth[column] = textWidth(header()->fontMetrics(), headerItem()->text(column)) + 10;
         minimumWidthTotal += minimumWidth[column];
 
@@ -1864,8 +1970,9 @@ void Playlist::slotUpdateColumnWidths()
     // useful weight to use as a divisor for each column's weight.
 
     double totalWeight = 0;
-    foreach(int column, visibleColumns)
+    for(int column : qAsConst(visibleColumns)) {
         totalWeight += m_columnWeights[column];
+    }
 
     // This can happen during startup, before we have the tracks loaded.
     if(qFuzzyIsNull(totalWeight)) {
@@ -1876,8 +1983,9 @@ void Playlist::slotUpdateColumnWidths()
     // width if we didn't have to handle the cases of minimum and maximum widths.
 
     QVector<int> weightedWidth(columnCount(), 0);
-    foreach(int column, visibleColumns)
+    for(int column : qAsConst(visibleColumns)) {
         weightedWidth[column] = int(double(m_columnWeights[column]) / totalWeight * viewport()->width() + 0.5);
+    }
 
     // The "extra" width for each column.  This is the weighted width less the
     // minimum width or zero if the minimum width is greater than the weighted
@@ -1907,7 +2015,7 @@ void Playlist::slotUpdateColumnWidths()
 
     // Fill in the values discussed above.
 
-    foreach(int column, visibleColumns) {
+    for(int column : qAsConst(visibleColumns)) {
         if(weightedWidth[column] < minimumWidth[column]) {
             readjust = true;
             extraWidth[column] = 0;
@@ -1933,7 +2041,7 @@ void Playlist::slotUpdateColumnWidths()
     // than the minimum widths, just use those, otherwise use the "readjusted
     // weighted width".
 
-    foreach(int column, visibleColumns) {
+    for(int column : qAsConst(visibleColumns)) {
         int width;
         if(readjust) {
             int adjustedExtraWidth = int(double(extraWidth[column]) * adjustmentRatio + 0.5);
@@ -1970,43 +2078,7 @@ void Playlist::slotShowRMBMenu(const QPoint &point)
     // Create the RMB menu on demand.
 
     if(!m_rmbMenu) {
-
-        // Probably more of these actions should be ported over to using KActions.
-
-        m_rmbMenu = new QMenu(this);
-
-        m_rmbMenu->addAction(QIcon::fromTheme("go-jump-today"),
-            i18n("Add to Play Queue"), this, SLOT(slotAddToUpcoming()));
-        m_rmbMenu->addSeparator();
-
-        if(!readOnly()) {
-            m_rmbMenu->addAction( action("edit_cut") );
-            m_rmbMenu->addAction( action("edit_copy") );
-            m_rmbMenu->addAction( action("edit_paste") );
-            m_rmbMenu->addSeparator();
-            m_rmbMenu->addAction( action("removeFromPlaylist") );
-        }
-        else
-            m_rmbMenu->addAction( action("edit_copy") );
-
-        m_rmbEdit = m_rmbMenu->addAction(i18n("Edit"));
-
-        m_rmbMenu->addAction( action("refresh") );
-        m_rmbMenu->addAction( action("removeItem") );
-
-        m_rmbMenu->addSeparator();
-
-        m_rmbMenu->addAction( action("guessTag") );
-        m_rmbMenu->addAction( action("renameFile") );
-
-        m_rmbMenu->addAction( action("coverManager") );
-
-        m_rmbMenu->addSeparator();
-
-        m_rmbMenu->addAction(
-            QIcon::fromTheme("folder-new"),
-            i18n("Create Playlist From Selected Items..."),
-            this, SLOT(slotCreateGroup()));
+        createPlaylistRMBMenu();
     }
 
     // Ignore any columns added by subclasses.
