@@ -233,7 +233,7 @@ public:
      * Set item to be the playing item.  If \a item is null then this will clear
      * the playing indicator.
      */
-    static void setPlaying(PlaylistItem *item, bool addToHistory = true);
+    void setPlaying(PlaylistItem *item, bool addToHistory = true);
 
     /**
      * Returns true if this playlist is currently playing.
@@ -364,6 +364,11 @@ public slots:
      */
     virtual void slotRefresh();
 
+    /**
+     * Opens the containing folder of the selected files.
+     */
+    virtual void slotOpenItemDir();
+
     void slotGuessTagInfo(TagGuesser::Type type);
 
     /**
@@ -551,6 +556,9 @@ private:
      */
     int leftMostVisibleColumn() const;
 
+    /// Creates the context menu on demand
+    void createPlaylistRMBMenu();
+
     /**
      * This method is used internally to provide the backend to the other item
      * lists.
